﻿/*--------------------------------------------------------------------------------
 * Adaptive Intelligence - Foundation Framework
 * Version 2.10
 * 
 * POP3 NOOP Command Class
 *      Represents the no-operation (NOOP) command.
 *------------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------------
 * System Imports
 *------------------------------------------------------------------------------*/
using System;
using Adaptive.Foundation;
using Adaptive.Foundation.Network.Cnr;
using Adaptive.Network.Sockets;

namespace Adaptive.Foundation.Network.Pop3
{
    /// <summary>
    /// Represents the no-operation (NOOP) command.
    /// </summary>
    public class NoopCommand : CnrSocketsCommandBase
    {
        /*--------------------------------------------------------------------------------
         * Constructor / Dispose / Destructor Methods
         *------------------------------------------------------------------------------*/
        #region Constructor / Dispose / Destructor Methods
        /// <summary>
        /// Initializes a new instance of the class.
        /// </summary>
        /// <param name="client">
        /// The Sockets client instance used for the command.
        /// </param>
        public NoopCommand(SocketsClient client) : base(client, "NOOP", (int)Pop3Commands.Noop)
        {
        }
        #endregion

        /*--------------------------------------------------------------------------------
         * Protected Methods / Event Methods
         *------------------------------------------------------------------------------*/
        #region Protected Methods / Event Methods
        /// <summary>
        /// Executes the command.
        /// </summary>
        /// <returns>
        /// A <see cref="CnrResponseBase"/> instance containing the response to the command execution.
        /// </returns>
        public override CnrResponseBase Execute()
        {
            Pop3Response response = null;
            string data = RenderCommand() + "\r\n";
            string result = string.Empty;

            result = Client.SendAndReceive(data, "\r\n", 30);
            if ((result != null) && (result.Length > 0))
            {
                response = new Pop3Response(result);
            }
            else
            {
                response = new Pop3Response();
            }

            return response;
        }
        /// <summary>
        /// Executes the command using the specified object.
        /// </summary>
        /// <param name="executionContextItem">
        /// An object containing data and/or used to execute the command.
        /// </param>
        /// <returns>
        /// A <see cref="CnrResponseBase"/> instance containing the response to the command execution.
        /// </returns>
        public override CnrResponseBase Execute(object executionContextItem)
        {
            return Execute();
        }
        /// <summary>
        /// Renders the command as a string.
        /// </summary>
        /// <returns>
        /// A string containing the command and parameter values.
        /// </returns>
        protected override string RenderCommand()
        {
            return "NOOP";
        }
        /// <summary>
        /// Renders the command as a string.
        /// </summary>
        /// <returns>
        /// A string containing the command and parameter values.
        /// </returns>
        /// <param name="renderer">
        /// A <see cref="CnrDataRendererBase"/> instance used to render the command.
        /// </param>
        protected override string RenderCommand(CnrDataRendererBase renderer)
        {
            return ((Pop3Renderer)renderer).RenderPop3Command(Pop3Commands.Noop);
        }
        /// <summary>
        /// Parses the original data content into the command object.
        /// </summary>
        /// <param name="parser">
        /// A <see cref="CnrDataParserBase"/> instance used to parse the command text.
        /// </param>
        /// <param name="originalData">
        /// The original data to be parsed.
        /// </param>
        protected override void ParseCommand(CnrDataParserBase parser, string originalData)
        {
        }
        #endregion

        /*--------------------------------------------------------------------------------
         * Public Methods / Functions
         *------------------------------------------------------------------------------*/
        #region Public Methods / Functions
        /// <summary>
        /// Clears the contents of the object instance without disposing of the instance.
        /// </summary>
        public override void Clear()
        {
        }
        #endregion
    }
}